@@ -18,22 +18,27 @@
 
 ### write data from a stream to a file
 
-write_stream <- function(dsd, file, n=100, block=100000L,
-  class=FALSE, sep=",", 
-  col.names=FALSE, row.names=FALSE, ...) UseMethod("write_stream")
+write_stream <- function(dsd, file, n=100, block=100000L, 
+  class=FALSE, append = FALSE, sep=",", 
+  header=FALSE, row.names=FALSE, ...) UseMethod("write_stream")
 
 write_stream.default <- function(dsd, file, n=100, block=100000L, 
-  class=FALSE, sep=",", col.names=FALSE, row.names=FALSE, ...) {
+  class=FALSE, append = FALSE, sep=",", header=FALSE, row.names=FALSE, ...) {
   stop(gettextf("write_stream not implemented for class '%s'.", class(dsd)))
 }
 
 write_stream.DSD <- function(dsd, file, n=100, block=100000L, 
-  class=FALSE, 
-  sep=",", col.names=FALSE, row.names=FALSE, ...) {	
+  class=FALSE, append = FALSE, sep=",", header=FALSE, row.names=FALSE, ...) {	
+
+  # make sure files are not overwritten
+  if(is(file, "character") && file.exists(file) && !append) 
+    stop("file exists already. Please remove the file first.")
   
   # string w/ file name (clears the file)
-  if (is(file, "character")) file <- file(file, open="w")
-  
+  if (is(file, "character")) {
+    if(append) file <- file(file, open="a")
+    else file <- file(file, open="w")
+  }
   # error	
   else if (!is(file, "connection")) stop("Please pass a valid connection!")
   
@@ -44,9 +49,10 @@ write_stream.DSD <- function(dsd, file, n=100, block=100000L,
   for (bl in .make_block(n, block)) {
     p <- get_points(dsd, bl, class=class)
     
-    write.table(p, 
-      file, sep=sep, append=TRUE, col.names=FALSE,
-      row.names=row.names, ...)
+    ## suppress warning for append and col.names
+    suppressWarnings(write.table(p, 
+      file, sep=sep, append=TRUE, col.names=header,
+      row.names=row.names, ...))
   }
   close(file)
 }
